% Demonstrate DTW on a simple toy example
% DTW code to be downladed from Dan Ellis' website at Columbia
% http://labrosa.ee.columbia.edu/matlab/dtw/
clear
close all
clc

path('..\..\voicebox',path);

%% Load utterances and compute MFCCs

[x1 fs] = wavread('1fast.wav'); 
[x2 fs] = wavread('1slow.wav'); 
[x3 fs] = wavread('7fast.wav'); 

mfcc1 = melcepst(x1, fs);
mfcc2 = melcepst(x2, fs);
mfcc3 = melcepst(x3, fs);

figure(1)
subplot(311); imagesc(mfcc1', [-3 6]); colorbar; title('ONE fast')
subplot(312); imagesc(mfcc2', [-3 6]); colorbar; title('ONE slow')
subplot(313); imagesc(mfcc3', [-3 6]); colorbar; title('SEVEN fast')

%% Align the slow and fast "ONES"
M = simmx(mfcc1',mfcc2');
[p, q] = dp(1-M);

figure(2)
subplot(121)
imagesc((1-M)');
colormap gray; colorbar; xlabel('source'); ylabel('target'); hold on
plot(round(p),round(q), 'LineWidth', 2)
drawnow

figure (3)
subplot(221); imagesc(mfcc1(p,:)', [-3 6]); colorbar; title('ONE fast aligned')
subplot(223); imagesc(mfcc2(q,:)', [-3 6]); colorbar; title('ONE slow aligned')

sum(M([p,q]))/length(p)

%% Align the slow "ONE" and "SEVEN"
M = simmx(mfcc1',mfcc3');
[p, q] = dp(1-M);

figure(2)
subplot(122)
imagesc((1-M)');
colormap gray; colorbar; xlabel('source'); ylabel('target'); hold on
plot(round(p),round(q), 'LineWidth', 2)
drawnow

sum(M([p,q]))/length(p)

figure(3)
subplot(222); imagesc(mfcc1(p,:)', [-3 6]); colorbar; title('ONE fast aligned')
subplot(224); imagesc(mfcc3(q,:)', [-3 6]); colorbar; title('SEVEN fast aligned')
